// Copyright (c) 2018, Adetu Ridwan and Egbesola Anthony.
import 'dart:io';
import 'dart:core';

/// A date utility interface to get every information of dates, months, weeks and year taking leap
/// year into consideration.
class DateUtilities {
  var dayOfWeek = 0;

  String? day(int length) {
    final day = <String>[
      'Saturday',
      'Sunday',
      'Monday',
      'Tuesday',
      'Wednesday',
      'Thursday',
      'Friday'
    ];

    var count = 0;
    String? resultDay;

    for (var counter = 1; counter <= length; counter++) {
      final check = ((counter > 639798) && (counter < 639811));
      if (check == true) {
      } else {
        if (count >= 7) {
          if (count % 7 == 0) count = 0;
        }
        resultDay = day[count];
        count++;
      }
    }
    if (count == 1)
      dayOfWeek = 7;
    else
      dayOfWeek = (count - 1);
    return resultDay;
  }

  /// It returns the name of the month for the given month number
  ///
  /// Args:
  ///   monthNum (int): The number of the month to get the name of.
  String month(final int monthNum) {
    final month = <String>[
      'January',
      'February',
      'March',
      'April',
      'May',
      'June',
      'July',
      'August',
      'September',
      'October',
      'November',
      'December'
    ];
    return month[monthNum - 1];
  }

  /// "Given a month number and a year, return the number of days in that month."
  ///
  /// The first line of the function is a comment. Comments are ignored by the Dart compiler
  ///
  /// Args:
  ///   monthNum (int): The month number, 0 for January, 1 for February, and so on.
  ///   year (int): The year to check.
  int? daysInMonth(final int monthNum, final int year) {
    if (monthNum > 12) {
      return null;
    }
    List<int> monthLength = List.filled(12, 0);
    monthLength[0] = 31;
    monthLength[2] = 31;
    monthLength[4] = 31;
    monthLength[6] = 31;
    monthLength[7] = 31;
    monthLength[9] = 31;
    monthLength[11] = 31;
    monthLength[3] = 30;
    monthLength[8] = 30;
    monthLength[5] = 30;
    monthLength[10] = 30;

    if (leapYear(year) == true)
      monthLength[1] = 29;
    else
      monthLength[1] = 28;

    return monthLength[monthNum - 1];
  }

  int getMonthGivenDayOfYear(int days, int year) {
    assert(days > 0);
    List<int> monthLength = List.filled(12, 0);
    monthLength[0] = 31;
    monthLength[2] = 31;
    monthLength[4] = 31;
    monthLength[6] = 31;
    monthLength[7] = 31;
    monthLength[9] = 31;
    monthLength[11] = 31;
    monthLength[3] = 30;
    monthLength[8] = 30;
    monthLength[5] = 30;
    monthLength[10] = 30;

    if (leapYear(year) == true)
      monthLength[1] = 29;
    else
      monthLength[1] = 28;

    for (var i = 0; i < monthLength.length; i++) {
      var daysInMonthLength = monthLength[i];
      if (days - daysInMonthLength < 0) return i;
    }

    return 12;
  }

  /// It returns the number of days past in the year.
  ///
  /// Args:
  ///   monthNum (int): The month number (1-12)
  ///   dayNum (int): The day of the month (1-31)
  ///   year (int): The year in question.
  int daysInYear(int year) => leapYear(year) ? 366 : 365;

  /// It returns the number of days past in the year.
  ///
  /// Args:
  ///   monthNum (int): The month number (1-12)
  ///   dayNum (int): The day of the month (1-31)
  ///   year (int): The year in question.
  int daysPastInYear(final int monthNum, final int dayNum, final int year) {
    var totalMonthLength = 0;
    for (var count = 1; count < monthNum; count++) {
      totalMonthLength += daysInMonth(count, year)!;
    }
    return totalMonthLength + dayNum;
  }

  /// It returns the number of days left in the year.
  ///
  /// Args:
  ///   monthNum (int): The number of the month (1-12)
  ///   dayNum (int): The day of the month (1-31)
  ///   year (int): The year in question.
  int daysLeftInYear(final int monthNum, final int dayNum, final int year) {
    var totalMonthLength = 0;
    for (var count = 1; count < monthNum; count++) {
      totalMonthLength += daysInMonth(count, year)!;
    }
    return totalMonthLength + dayNum - daysInYear(year);
  }

  /// Prints the calendar for the given month and year.
  ///
  /// Args:
  ///   monthNum (int): The number of the month to print.
  ///   year (int): The year to print the calendar for.
  void printMonthCalendar(final int monthNum, final int year) {
    int dayNum = 1;
    final str_Day = <String>['Sun', 'Mon', 'Tue', 'Wed', 'Thur', 'Fri', 'Sat'];
    int dayDays = 1;

    for (int i = 0; i < 7; i++) {
      stdout.write('${str_Day[i]}\t');
    }
    stdout.writeln();
    for (int i = 1; i <= 6; i++) {
      for (int j = 1; j <= 7; j++) {
        if (dayNum >= dayOfWeek) {
          if (dayDays <= daysInMonth(monthNum, year)!) {
            stdout.write('${dayDays}\t');
          }
          ++dayDays;
        } else if (dayNum < dayOfWeek) stdout.write('\t');

        dayNum++;
      }
      stdout.writeln();
    }
  }

  /// It returns the week number of the year.
  ///
  /// Args:
  ///   monthNum (int): The month number (1-12)
  ///   dayNum (int): The day of the month (1-31)
  ///   year (int): The year of the date you want to get the week of.
  int getWeek(int monthNum, int dayNum, int year) {
    double a = (daysPastInYear(monthNum, dayNum, year) / 7) + 1;
    return a.toInt();
  }

  /// Given a year, return true if it's a leap year, false otherwise.
  ///
  /// Args:
  ///   year (int): the year to check
  bool leapYear(int year) {
    bool leapYear = false;
    bool leap = ((year % 100 == 0) && (year % 400 != 0));
    if (leap == true)
      leapYear = false;
    else if (year % 4 == 0) leapYear = true;

    return leapYear;
  }
}
